%%%%%%%%%%%%%%%%%%%%%DFT - DEMO 1%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In this demo, we will see how to reconstruct a simple image from its
% Fourier coefficients.
% Class - ECE 189
% Instructor - Dr. Rebecca Willett
% Author - Kalyani Krishnamurthy (09/03/08)
% Send comments to: kk63@ee.duke.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc; clear all;
% Define the size of the input image
M = 16;
N = 16;
% Form a simple test image
x = zeros(M,N);
x(2:8,4:8) = 1;
x(5:7,6:7) = 0;
figure(1); imagesc(x); axis image; colormap gray; title('x'); colorbar;
set(gca,'FontSize',16)

% Compute the two dimensional Fourier transform of the input image.
% fftshift - just to shift the (0,0) frequency to the center for easy
% visualization.
X = fft2(x);
figure(2); imagesc(abs(fftshift(X))); axis image; colormap gray; title('fft2(x)'); colorbar;
set(gca,'FontSize',16)
% Sort the fft coefficients by their absolute values.
[sort_fft_coeff,sort_ind] = sort(abs(X(:)),'descend');
% Initialize the estimate of f to be zeros.
x_hat_prev = zeros(M,N);

% Create a grid rows and columns corresponding to the spatial intensity
% image (in terms of x and y)
[n,m] = meshgrid(0:N-1,0:M-1); 

% Loop through the Fourier coefficients (in the descending order of the
% absolute value of the Fourier coefficients) and reconstruct f from them.
for ii = 1:length(sort_fft_coeff(:))
   % Find the frequency locations m_arr and n_arr corresponding to the Fourier
   % coefficient. 
   [k_arr, l_arr] = ind2sub(size(x),sort_ind(ii));
   k_arr = k_arr-1;k_arr_s = k_arr-M.*(k_arr>=M/2);
   l_arr = l_arr-1;l_arr_s = l_arr-N.*(l_arr>=N/2);
   temp = (2*pi*1j).*((k_arr.*m/M) + (l_arr.*n/N));
   X_kl = X(sort_ind(ii));
   ith_component = X_kl*exp(temp)/M/N;
   x_hat = x_hat_prev+ith_component;
   
   figure(2);clf;
   imagesc(abs((X)));axis image;colormap gray;
   hold on;   scatter(l_arr+1,k_arr+1,100,'r','s','filled');
   title('|X|')
   
   figure(20);clf;
   imagesc((0:(N-1))-N/2,(0:(M-1))-M/2,abs(fftshift(X)));axis image;colormap gray;
   hold on;   scatter(l_arr_s,k_arr_s,100,'r','s','filled');
   title('fftshift|X|')
   
   
   figure(5); imagesc(abs(x_hat_prev)); axis image; colormap gray; colorbar;
   title(['$|\hat x_{' num2str(ii-1) '}|$'],'Interpreter','latex');
   set(gca,'FontSize',16)
   figure(6); imagesc(real(ith_component)); axis image; ...
      colormap gray; colorbar;
   title({['$|e^{j2\pi (m \times ' num2str(k_arr) '/M + n \times ' num2str(l_arr) ...
      '/N)}|$, where $|X[' num2str(k_arr) ',' num2str(l_arr)...
      ']| = ',num2str(sort_fft_coeff(ii)) ...
      '; k = ' num2str(k_arr) ', l = ' num2str(l_arr) '$']},...
      'Interpreter','latex');
   set(gca,'FontSize',16)
   figure(7); imagesc(abs(x_hat)); axis image; colormap gray; colorbar;
   title(['$|\hat x_{' num2str(ii) '}|$'],'Interpreter','latex');
   disp(['ii = ' num2str(ii) ', press any key...']);
   set(gca,'FontSize',16)
   pause;
   
   x_hat_prev = x_hat;
end